==> sdk-1.0.32-sources.jar: main/nashid/verify/sdk/viewmodel/SkipNfcLiveNessViewModel.kt <==
package nashid.verify.sdk.viewmodel

import android.app.Application
import android.graphics.Bitmap
import androidx.lifecycle.ViewModel
import nashid.verify.sdk.VerifySDKManager
import nashid.verify.sdk.data.repository.SkipLiveNessRepository
import nashid.verify.sdk.model.DocumentVerification
import nashid.verify.sdk.model.FailureReason
import nashid.verify.sdk.model.ScanDocumentResult
import nashid.verify.sdk.utils.ArtifactType
import nashid.verify.sdk.utils.OtherConstant
import nashid.verify.sdk.utils.SdkConfig
import nashid.verify.sdk.utils.Utility

class SkipNfcLiveNessViewModel(
    private val skipNfcLiveNessLiveData: SkipNfcLiveNessLiveData,
    private val skipLiveNessRepository: SkipLiveNessRepository,
    private val context: Application,
    private val scanDocumentViewModel: ScanDocumentViewModel,
) : ViewModel() {
    init {
        observeScanDocumentViewModel()
    }

    private fun observeScanDocumentViewModel() {
        scanDocumentViewModel.result.observeForever { result ->
            if (result is ScanDocumentResult.FileKeyResult) {
                Utility.getInstance().saveFileKeyAndContinue(result.fileKey, result.artifactType)
            }
            if (result is ScanDocumentResult.Error) {
                skipLiveNessRepository.callToComparison()
            }
        }
    }

    fun handleInternetConnectionData(isInternetAvailable: Boolean) {
        skipNfcLiveNessLiveData.setInternetConnection(isInternetAvailable)
        if (!isInternetAvailable) {
            skipNfcLiveNessLiveData.setInternetConnection(false)
        } else {
            skipNfcLiveNessLiveData.setIsApiCalled(true)
            if (Utility.getInstance().ocrScanningFailed()) {
                skipLiveNessRepository.callToComparison()
            } else if (!SdkConfig.isFaceMatchingEnabled && !SdkConfig.isPassiveLiveNessEnabled) {
                skipLiveNessRepository.callToComparison()
            } else {
                skipNfcLiveNessLiveData.setIsLiveness(true)
            }
        }
    }

    fun getLiveNessData() = skipNfcLiveNessLiveData

    fun uploadArtifact(
        bitmap: Bitmap?,
        artifactType: ArtifactType,
    ) {
        val file = Utility.getInstance().bitmapToFile(bitmap!!, context)
        val filePart = file?.let { Utility.getInstance().createMultipartBody(it) }
        if (filePart != null) {
            scanDocumentViewModel.uploadArtifact(
                SdkConfig.registerToken!!,
                Utility.getInstance().getAppKey(),
                filePart,
                artifactType.type,
            )
        }
    }

    fun updateArtifacts() {
        scanDocumentViewModel.updateRequiredArtifacts()
    }

    private fun submitVerification(verification: DocumentVerification) {
        VerifySDKManager.getInstance().submitVerification(verification)
    }

    private fun failedArtifactsVerification(verification: DocumentVerification) {
        VerifySDKManager.getInstance().failedArtifactsVerification(verification)
    }

    fun validateJustCompletion() {
        skipLiveNessRepository.callToComparison()
    }

    fun submitVerification(
        latitude: Double,
        longitude: Double,
        isShowStartScanning: Boolean,
    ) {
        prepareResultData(
            latitude,
            longitude,
            isShowStartScanning,
            callback =
                object : DocumentVerificationCallback {
                    override fun onResult(result: DocumentVerification) {
                        submitVerification(result)
                    }
                },
        )
    }

    private fun prepareResultData(
        latitude: Double,
        longitude: Double,
        isShowStartScanning: Boolean,
        callback: DocumentVerificationCallback,
    ) {
        var isFlowCompleted = true
        if (isShowStartScanning) {
            skipNfcLiveNessLiveData.setIsLiveness(false)
            isFlowCompleted = false
        }
        val documentType = SdkConfig.viewType.value
        skipLiveNessRepository.createMetaData(latitude, longitude) { metaData ->
            val utility = Utility.getInstance()
            val isNFCEnabled = SdkConfig.isNfcScanningEnabled && !OtherConstant.getInstance().skipNFC
            val data = skipLiveNessRepository.createData(isNFCEnabled, documentType, skipNfcLiveNessLiveData.getIsLiveness().value)
            val artifacts = skipLiveNessRepository.handleArtifacts()
            val isOcrFailed = utility.ocrScanningFailed()
            val failureReasons =
                mutableListOf<String>().apply {
                    when {
                        utility.hasOcrMaxRetries() -> {
                            add(FailureReason.OCR_MAX_RETRIES.name)
                        }

                        utility.hasNfcMaxRetries() -> {
                            add(FailureReason.NFC_MAX_RETRIES.name)
                        }

                        utility.hasPassiveLivenessMaxRetries() -> {
                            add(FailureReason.PASSIVE_LIVENESS_MAX_RETRIES.name)
                        }

                        utility.hasActiveLivenessMaxRetries() -> {
                            add(FailureReason.ACTIVE_LIVENESS_MAX_RETRIES.name)
                        }
                    }
                    if (isEmpty() && isOcrFailed) {
                        add(FailureReason.GENERAL_ERROR.name)
                    }
                }
            val statusNote = failureReasons.joinToString(",")
            val statusValue =
                when {
                    utility.hasOcrMaxRetries() ||
                        utility.hasNfcMaxRetries() ||
                        utility.hasActiveLivenessMaxRetries() ||
                        utility.hasPassiveLivenessMaxRetries() -> 2
                    isFlowCompleted -> 1
                    else -> 2
                }
            val result =
                DocumentVerification(
                    metaDataModel = metaData,
                    type = documentType,
                    data = data,
                    artifacts = artifacts,
                    status = statusValue,
                    statusNote = if (statusValue == 2) statusNote else null,
                )
            callback.onResult(result)
        }
    }

    override fun onCleared() {
        super.onCleared()
        skipNfcLiveNessLiveData.clear()
    }

    interface DocumentVerificationCallback {
        fun onResult(result: DocumentVerification)
    }

    fun missingArtifactsVerification(
        latitude: Double,
        longitude: Double,
        isShowStartScanning: Boolean,
    ) {
        prepareResultData(
            latitude,
            longitude,
            isShowStartScanning,
            callback =
                object : DocumentVerificationCallback {
                    override fun onResult(verification: DocumentVerification) {
                        failedArtifactsVerification(verification)
                    }
                },
        )
    }
}

==> sdk-1.0.35-sources.jar: main/nashid/verify/sdk/viewmodel/SkipNfcLiveNessViewModel.kt <==
package nashid.verify.sdk.viewmodel

import android.app.Application
import android.graphics.Bitmap
import androidx.lifecycle.ViewModel
import nashid.verify.sdk.VerifySDKManager
import nashid.verify.sdk.data.repository.SkipLiveNessRepository
import nashid.verify.sdk.model.DocumentVerification
import nashid.verify.sdk.model.FailureReason
import nashid.verify.sdk.model.ScanDocumentResult
import nashid.verify.sdk.utils.ArtifactType
import nashid.verify.sdk.utils.OtherConstant
import nashid.verify.sdk.utils.SdkConfig
import nashid.verify.sdk.utils.Utility

class SkipNfcLiveNessViewModel(
    private val skipNfcLiveNessLiveData: SkipNfcLiveNessLiveData,
    private val skipLiveNessRepository: SkipLiveNessRepository,
    private val context: Application,
    private val scanDocumentViewModel: ScanDocumentViewModel,
) : ViewModel() {
    init {
        observeScanDocumentViewModel()
    }

    private fun observeScanDocumentViewModel() {
        scanDocumentViewModel.result.observeForever { result ->
            if (result is ScanDocumentResult.FileKeyResult) {
                Utility.getInstance().saveFileKeyAndContinue(result.fileKey, result.artifactType)
            }
            if (result is ScanDocumentResult.Error) {
                skipLiveNessRepository.callToComparison()
            }
        }
    }

    fun handleInternetConnectionData(isInternetAvailable: Boolean) {
        skipNfcLiveNessLiveData.setInternetConnection(isInternetAvailable)
        if (!isInternetAvailable) {
            skipNfcLiveNessLiveData.setInternetConnection(false)
        } else {
            skipNfcLiveNessLiveData.setIsApiCalled(true)
            if (Utility.getInstance().ocrScanningFailed()) {
                skipLiveNessRepository.callToComparison()
            } else if (SdkConfig.isActiveLiveNessEnabled || SdkConfig.isPassiveLiveNessEnabled) {
                skipNfcLiveNessLiveData.setIsLiveness(true)
            } else {
                skipLiveNessRepository.callToComparison()
            }
        }
    }

    fun getLiveNessData() = skipNfcLiveNessLiveData

    fun getScanDocumentViewModel() = scanDocumentViewModel

    fun uploadArtifact(
        bitmap: Bitmap?,
        artifactType: ArtifactType,
    ) {
        val file = Utility.getInstance().bitmapToFile(bitmap!!, context)
        val filePart = file?.let { Utility.getInstance().createMultipartBody(it) }
        if (filePart != null) {
            scanDocumentViewModel.uploadArtifact(
                SdkConfig.registerToken!!,
                Utility.getInstance().getAppKey(),
                filePart,
                artifactType.type,
            )
        }
    }

    fun updateArtifacts() {
        scanDocumentViewModel.updateRequiredArtifacts()
    }

    private fun submitVerification(verification: DocumentVerification) {
        VerifySDKManager.getInstance().submitVerification(verification)
    }

    private fun failedArtifactsVerification(verification: DocumentVerification) {
        VerifySDKManager.getInstance().failedArtifactsVerification(verification)
    }

    fun validateJustCompletion() {
        skipLiveNessRepository.callToComparison()
    }

    fun submitVerification(
        latitude: Double,
        longitude: Double,
        isShowStartScanning: Boolean,
    ) {
        prepareResultData(
            latitude,
            longitude,
            isShowStartScanning,
            callback =
                object : DocumentVerificationCallback {
                    override fun onResult(result: DocumentVerification) {
                        submitVerification(result)
                    }
                },
        )
    }

    private fun prepareResultData(
        latitude: Double,
        longitude: Double,
        isShowStartScanning: Boolean,
        callback: DocumentVerificationCallback,
    ) {
        var isFlowCompleted = true
        if (isShowStartScanning) {
            skipNfcLiveNessLiveData.setIsLiveness(false)
            isFlowCompleted = false
        }
        val documentType = SdkConfig.viewType.value
        skipLiveNessRepository.createMetaData(latitude, longitude) { metaData ->
            val utility = Utility.getInstance()
            val isNFCEnabled = SdkConfig.isNfcScanningEnabled && !OtherConstant.getInstance().skipNFC
            val data = skipLiveNessRepository.createData(isNFCEnabled, documentType, skipNfcLiveNessLiveData.getIsLiveness().value)
            val artifacts = skipLiveNessRepository.handleArtifacts()
            val isOcrFailed = utility.ocrScanningFailed()
            val failureReasons =
                mutableListOf<String>().apply {
                    when {
                        utility.hasOcrMaxRetries() -> {
                            add(FailureReason.OCR_MAX_RETRIES.name)
                        }

                        utility.hasNfcMaxRetries() -> {
                            add(FailureReason.NFC_MAX_RETRIES.name)
                        }

                        utility.hasPassiveLivenessMaxRetries() -> {
                            add(FailureReason.PASSIVE_LIVENESS_MAX_RETRIES.name)
                        }

                        utility.hasActiveLivenessMaxRetries() -> {
                            add(FailureReason.ACTIVE_LIVENESS_MAX_RETRIES.name)
                        }
                    }
                    if (isEmpty() && isOcrFailed) {
                        add(FailureReason.GENERAL_ERROR.name)
                    }
                }
            val statusNote = failureReasons.joinToString(",")
            val statusValue =
                when {
                    utility.hasOcrMaxRetries() ||
                        utility.hasNfcMaxRetries() ||
                        utility.hasActiveLivenessMaxRetries() ||
                        utility.hasPassiveLivenessMaxRetries() -> 2
                    isFlowCompleted -> 1
                    else -> 2
                }
            val result =
                DocumentVerification(
                    metaDataModel = metaData,
                    type = documentType,
                    data = data,
                    artifacts = artifacts,
                    status = statusValue,
                    statusNote = if (statusValue == 2) statusNote else null,
                )
            callback.onResult(result)
        }
    }

    override fun onCleared() {
        super.onCleared()
        skipNfcLiveNessLiveData.clear()
    }

    interface DocumentVerificationCallback {
        fun onResult(result: DocumentVerification)
    }

    fun missingArtifactsVerification(
        latitude: Double,
        longitude: Double,
        isShowStartScanning: Boolean,
    ) {
        prepareResultData(
            latitude,
            longitude,
            isShowStartScanning,
            callback =
                object : DocumentVerificationCallback {
                    override fun onResult(verification: DocumentVerification) {
                        failedArtifactsVerification(verification)
                    }
                },
        )
    }
}
